local corpse = class{name = 'corpse', function(self, dir, x,y)
	self.direction = dir
	self.pos = vector(x,y)
	self.anim = newAnimation(IMG['Dead_zombie.png'], 16,20,.1,4)
	self.anim:setMode('once')
	Entities.register(self)
end}

function corpse:draw()
	love.graphics.setColor(255,255,255)
	self.anim:draw(self.pos.x,self.pos.y, 0, self.direction,1, 3.5,10)
end

function corpse:update(dt)
	self.anim:update(dt)
end

local names = {
	'Carl', 'Lenny', 'Joe', 'James', 'Robert', 'Thomas', 'David',
	'Paul', 'Mark', 'Daniel', 'Steve', 'Brian',
}

Human = class{function(self, x,y)
	self.shape = HC.addPolygon(0,8, 3,4, 7,4, 10,8, 10,23, 0,23)
	self.shape:moveTo(x,y)
	self.shape.object = self
	HC.addToGroup('human', self.shape)

	self.velocity = vector(0,0)
	self.speed = math.random(40,55)
	self.life = 1
	self.endurance = math.random() * .2 + .9
	self.direction = 1
	self.falling = true
	self.anim = newAnimation(IMG['Guy_animation.png'], 16, 20, .15, 4)
	self.ammo = 0

	self.name = names[ math.random(1, #names) ]

	self.light = Lights.add(0,0, 150)

	self.weapon = NullWeapon()

	Entities.register(self)
end}
make_collidable(Human)

Human:register('zombie', function(self,other,dx,dy)
	local d = vector(dx,dy):normalize_inplace()
	self.velocity = self.velocity + vector(d.x*20,0) * self.speed
	self.falling = true
	self.shape:move(dx/10,dy/10)
	other.shape:move(-dx/10,-dy/10)
	self.life = self.life - love.timer.getDelta() / self.endurance
	-- TODO: blood
end)

Human:register('obstacle', function(self,other,dx,dy)
	self.obstacle_flag = self.obstacle_flag or math.abs(dx) > 0
end)

Human:register('item', function(self, other)
	HC.remove(other.shape)
	Entities.remove(other)
	Party.weapons[#Party.weapons+1] = other.weapon
end)

Human:register('goal', function(self, other)
	self:on_goal(other)
end)

function Human:start_follow(other)
	local text_entity = {text = "It's dangerous to be alone"}
	function text_entity:draw()
		local x,y = other.shape:center()
		local w = Glyphs.width(self.text, 5)
		love.graphics.setColor(255,255,255)
		Glyphs.print(self.text, (x + 8) - w/2, y - 20, 5, .2)
	end
	function text_entity:update() end
	Entities.register(text_entity)
	if #Party.weapons > 0 then
		Timer.add(2.5, function()
			text_entity.text = "Take this and come with me"
			-- give the guy a random weapon
			self.weapon = table.remove(Party.weapons, math.random(1, #Party.weapons))
		end)
	else
		Timer.add(2.5, function() text_entity.text = "Come with me" end)
	end
	Timer.add(5, function() Entities.remove(text_entity) end)
	Party[#Party+1] = self
end

local function player_control(self)
	self.walking = false
	local new_vel = vector(0,0)
	local dir = 0
	if love.keyboard.isDown('left') then
		dir = -1
	elseif love.keyboard.isDown('right') then
		dir = 1
	end

	if dir ~= 0 then
		new_vel.x = dir
		self.direction = dir
		self.walking = true
	end

	if not self.falling and love.keyboard.isDown('up') then
		new_vel.y = -2.5
		self.falling = true
	end

	if love.keyboard.isDown(' ') then
		self.weapon:shoot()
	end

	return new_vel
end

local function closest_human(self)
	local pos = vector(self.shape:center())
	local closest, dist = nil, 30 * 30
	for _,human in ipairs(Party) do
		if human ~= self then
			local target = vector(human.shape:center()) - pos
			local dsq = target:len2()
			if dist > dsq then
				closest, dist = human, dsq
			end
		end
	end
	return closest
end

local function closest_zombie(self)
	local pos = vector(self.shape:center())
	local closest, dist = nil, (self.light.radius * .7) ^ 2
	for _,zombie in pairs(Horde) do
		local target = vector(zombie.shape:center()) - pos
		local dsq = target:len2()
		if dist > dsq then
			closest, dist = target, dsq
		end
	end
	return closest, math.sqrt(dist)
end

local function ai_control(self)
	if not self.following then
		self.following = closest_human(self)
		if not self.following then
			return vector(0,0)
		else
			self:start_follow(self.following)
		end
	end

	local dir = vector(self.following.shape:center()) - vector(self.shape:center())
	local dist = dir:len()
	self.direction = dir.x > 0 and 1 or -1
	local new_vel = vector(0,0)
	if dist > 40 then
		if not self.falling and (self.obstacle_flag or dir.y/dir:len() > .5) then
			new_vel.y = -2.5
			self.falling = true
		end
		new_vel.x = dir.x > 0 and 1 or -1 -- TODO: fleeing flag
		self.direction = new_vel.x
	end
	self.obstacle_flag = false

	if not self.weapon.null_weapon then
		local direction, zombie_dist = closest_zombie(self)
		if direction and dist < 50 and math.abs(direction.y) < 10 then
			if zombie_dist > 20 then
				new_vel.x = 0
				self.direction = direction.x > 0 and 1 or -1
				self.weapon.direction = self.direction
				self.weapon:shoot()
			else
				self.direction = direction.x < 0 and 1 or -1
				if not self.falling and (self.obstacle_flag or dir.y/dir:len() > .5) then
					new_vel.y = -2.5
					self.falling = true
				end
			end
		end
	end

	self.walking = new_vel.x ~= 0

	return new_vel
end

function Human:update(dt)
--	if self.weapon.null_weapon and #Party.weapons >= 1 then
--		self.weapon = Party.weapons[1]
--		table.remove(Party.weapons,1)
--	end

	if self.life <= 0 then
		-- replace with dying animation
		HC.remove(self.shape)
		Entities.remove(self)
		corpse(self.direction, self.shape:center())
		self:on_die()
	end

	local new_vel
	if self.player_has_control then
		new_vel = player_control(self)
	else
		new_vel = ai_control(self)
	end

	new_vel = new_vel * self.speed

	if self.falling then
		new_vel = new_vel + vector(0,10)
	else
		self.velocity = self.velocity * .5
	end
	self.velocity = self.velocity + new_vel

	if self.velocity.x > self.speed then
		self.velocity.x = self.speed
	elseif self.velocity.x < -self.speed then
		self.velocity.x = -self.speed
	end

	if not self.falling then
		if self.walking then
			self.anim:update(dt)
		elseif not self.falling then
			self.anim:seek(1)
		end
	end

	self.shape:move(self.velocity * dt)
	self.light.pos = vector(self.shape:center())
	self.weapon.pos = vector(self.shape:center())
	self.weapon.direction = self.direction
	self.weapon.anim_offset = case (self.anim.position) {
		[2] = vector(0,-1), [3] = vector(1,-1), [4] = vector(1,0),
		default = vector(0,0)
	}
end

function Human:draw()
	local x,y = self.shape:center()
	love.graphics.setColor(255,255,255)
	self.anim:draw(x,y, 0, self.direction,1, 5,10)
	self.weapon:draw()
	if self.player_has_control then
		love.graphics.setColor(110,56,25, 50)
		love.graphics.circle('fill',x, y-18, 3, 4)
	end
end

function Human:draw_status(x,y)
	local w,h = IMG['Guy.png']:getWidth(), IMG['Guy.png']:getHeight()
	love.graphics.setColor(255,169,105,70)
	love.graphics.rectangle('fill', x,y, w+4,h+8)
	
	-- health bar
	love.graphics.setColor(255,0,0,120)
	love.graphics.rectangle('fill', x+2,y+2, w,3)
	love.graphics.setColor(0,120,0, 255)
	love.graphics.rectangle('fill', x+2,y+2, math.max(0,self.life)*w,3)

	-- icon
	love.graphics.setColor(255,255,255)
	love.graphics.draw(IMG['Guy.png'], x+w/2+2,y+h/2+3, 0, .7,.7, w/2,h/2)

	-- name
	Glyphs.print(self.name, x + (w - Glyphs.width(self.name,7))/2 + 2, y+h, 7, .4)
end
