local corpse = class{name = 'corpse', function(self, dir, x,y)
	self.direction = dir
	self.pos = vector(x,y)
	self.anim = newAnimation(IMG['Dead_zombie.png'], 16,20,.1,4)
	self.anim:setMode('once')
	Entities.register(self)
end}

function corpse:draw()
	love.graphics.setColor(255,255,255)
	self.anim:draw(self.pos.x,self.pos.y, 0, self.direction,1, 3.5,10)
end

function corpse:update(dt)
	self.anim:update(dt)
end

Zombie = class{name='Zombie',function(self, x,y)
	self.shape = HC.addRectangle(0,0, 7,20)
	self.shape:moveTo(x,y)
	self.shape.object = self
	HC.addToGroup('zombie', self.shape)

	self.velocity = vector(0,0)
	self.speed = math.random(15,25)
	self.sight = math.random(80,150)
	self.life = 1
	self.endurance = .9 + math.random() * .5
	self.direction = 1
	self.falling = true
	self.anim = newAnimation(IMG['Zombie_animation.png'], 16, 20, .1,4)

	Timer.addPeriodic(1, function() self.direction = math.random() > .5 and 1 or -1 end)

	Entities.register(self)
	Horde[self] = self
end}
make_collidable(Zombie)

Zombie:register('bullet', function(self, bullet)
	bullet:remove()
	self.life = self.life - bullet.damage / self.endurance
end)

function Zombie:update(dt)
	if self.life <= .0 then
		HC.remove(self.shape)
		Entities.remove(self)
		Horde[self] = nil
		corpse(self.direction, self.shape:center())
		return
	end

	if math.random() < .000001 then
		if math.random() > .5 then
			love.audio.play('snd/zombie_moan.ogg')
		else
			love.audio.play('snd/zombie_snarl.ogg')
		end
	end

	local new_vel = vector(0,0)

	local pos = vector(self.shape:center())
	local closest, dist = nil, self.sight * self.sight
	for _,human in ipairs(Party) do
		local target = vector(human.shape:center()) - pos
		local dsq = target:len2()
		if dist > dsq then
			closest, dist = target, dsq
		end
	end

	if dist < 150 * 150 then
		if math.random() < .001 then
			if math.random() > .5 then
				love.audio.play('snd/zombie_moan.ogg')
			else
				love.audio.play('snd/zombie_snarl.ogg')
			end
		end
	end

	if dist < self.sight*self.sight then
		self.direction = closest.x > 0 and 1 or -1
	end

	if self.falling then
		new_vel = new_vel + vector(0,10)
		self.walking = false
	else
		new_vel.x = self.direction * self.speed
		self.velocity = self.velocity * .5
		self.walking = true
	end
	self.velocity = self.velocity + new_vel

	if self.velocity.x > self.speed then
		self.velocity.x = self.speed
	elseif self.velocity.x < -self.speed then
		self.velocity.x = -self.speed
	end

	if not self.falling then
		if self.walking then
			self.anim:update(dt)
		else
			self.anim:seek(1)
		end
	end

	self.shape:move((self.velocity * dt):unpack())
end

function Zombie:draw()
	local x,y = self.shape:center()
	love.graphics.setColor(255,255,255)
	self.anim:draw(x,y, 0, self.direction,1, 3.5,10)

	love.graphics.setColor(255,0,0,200)
	love.graphics.rectangle('fill', x-8,y-14,16,2)

	love.graphics.setColor(0,128,0)
	love.graphics.rectangle('fill', x-8,y-14,math.max(0,self.life)*16,2)
end
