local entities = {
	foreground = {},
	background = {}
}
local to_remove = {}

local function register(obj, table)
	local table = table or 'foreground'
	entities[table][obj] = obj
end

local function remove(obj, table)
	to_remove[#to_remove+1] = {table = table or 'foreground', obj = obj}
end

function clear()
	entities = {
		foreground = {},
		background = {}
	}
end

local function call(func,...)
	for _, obj in pairs(entities.background) do
		obj[func](obj, ...)
	end
	for _, obj in pairs(entities.foreground) do
		obj[func](obj, ...)
	end
	for _,obj in ipairs(to_remove) do
		entities[obj.table][obj.obj] = nil
	end
	to_remove = {}
end

return {
	register = register,
	remove = remove,
	clear = clear,
	call = call,
	draw = function(...) call('draw', ...) end,
	update = function(...) call('update', ...) end,
}
