local mask_radius = 200
local sprite = love.image.newImageData(2*mask_radius, 2*mask_radius)
sprite:mapPixel(function(x,y)
	x = (x - mask_radius) / mask_radius
	y = (y - mask_radius) / mask_radius
	local a = math.pow(x*x + y*y, .55)
	return 255,255,255, math.max(0, 1-a) * 255
end)
sprite = love.graphics.newImage(sprite)
sprite:setFilter('linear', 'linear')

local is_ok, fb = pcall(love.graphics.newFramebuffer)
if not is_ok then
	is_ok, fb = pcall(love.graphics.newFramebuffer,
		nextPO2(love.graphics.getWidth()), nextPO2(love.graphics.getHeight()))
end
if not is_ok then
	error("no framebuffer support")
end

local lights = {}
local function add(x,y,radius)
	local light = {}
	light.pos = vector(x,y)
	light.scale = radius / mask_radius
	light.radius = radius
	
	lights[light] = light
	return light
end

local function remove(light)
	lights[light] = nil
end

local function clear()
	lights = {}
end

local function draw(cam)
	local ul = cam:toWorldCoords(vector(0,0))
	local lr = ul + vector(love.graphics.getWidth(), love.graphics.getHeight())

	love.graphics.setRenderTarget(fb)
	love.graphics.setColor(0,0,0)
	love.graphics.rectangle('fill', ul.x,ul.y,
			love.graphics.getWidth(), love.graphics.getHeight())
		
	love.graphics.setBlendMode('subtractive')
	for _,l in pairs(lights) do
		local scale = l.scale * (1 + (2 * math.random() - 1) * .1) -- flicker
		love.graphics.draw(sprite, l.pos.x,l.pos.y, 0,
			scale,scale, mask_radius, mask_radius)
	end

	love.graphics.setRenderTarget()
	love.graphics.setBlendMode('alpha')
	love.graphics.setColor(0,0,0)
	love.graphics.draw(fb,ul.x,ul.y, 0, 1/cam.zoom,1/cam.zoom)
end

return {
	add = add,
	remove = remove,
	clear = clear,
	draw = draw,
	lights = lights,
}
