GS       = require 'hump.gamestate'
HC       = require 'hardoncollider'
vector   = require 'hardoncollider.vector'
Timer    = require 'hump.timer'
Camera   = require 'hump.camera'
class    = require 'hardoncollider.class'
memoize  = require 'memoize'
Entities = require 'entities'
Glyphs   = require 'glyphs'
require 'AnAL'

function case(x)
	return function (of)
		local what = of[x] or of.default
		if type(what) == "function" then
			return what()
		end
		return what
	end
end

function nextPO2(x)
	return math.pow(2, math.floor(math.log(x)/math.log(2) + .5))
end
assert(nextPO2(3) == 4)

-- MAKE IMAGES NON INTERPOLATING WHEN SCALED
do
	local newImage = love.graphics.newImage
	function love.graphics.newImage(...)
		local img = newImage(...)
		img:setFilter('nearest', 'nearest')
		return img
	end
end

-- RESOURCE CACHING
love.graphics.setFont = memoize( love.graphics.setFont )

IMG = setmetatable({}, {__index = function(self, key)
	local res = love.graphics.newImage("img/" .. key)
	rawset(self, key, res)
	return res
end})

-- AUDIO MANAGER
do
	local sources = {}
	function love.audio.update()
		local remove = {}
		for _,s in pairs(sources) do
			if s:isStopped() then
				remove[#remove + 1] = s
			end
		end

		for i,s in ipairs(remove) do
			sources[s] = nil
		end
	end

	local play = love.audio.play
	function love.audio.play(what, how, loop)
		local src = what
		if type(what) ~= "userdata" or not what:typeOf("Source") then
			src = love.audio.newSource(what, how)
		end

		src:setLooping(loop or false)
		play(src)
		sources[src] = src
		return src
	end

	local stop = love.audio.stop
	function love.audio.stop(src)
		if not src then return end
		stop(src)
		sources[src] = nil
	end
end

-- STATE MAGIC
State = setmetatable({}, {__index = function(self, key)
	local state = love.filesystem.load( 'states/' .. key .. '.lua' )()
	rawset(self, key, state)
	return state
end})

-- HOLY TRINITY
function love.load()
	math.randomseed(os.time())
	love.mouse.setVisible(false)
	love.graphics.setBackgroundColor(0,0,0)
	GS.registerEvents()

	GS.switch(State.menu)
	love.graphics.setFont(14)
end

function love.update(dt)
	Timer.update(dt)
	love.audio.update()
end

function love.draw()
	love.graphics.setColor(255,255,255)
	love.graphics.print("FPS: " .. love.timer.getFPS(), 10,10)
end
