Lights = require 'lights'
require 'collidable'
require 'actors.obstacle'
require 'actors.human'
require 'actors.zombie'
require 'actors.bullet'
require 'actors.weapon'
require 'actors.item'
require 'actors.asset'
local RB = require 'hump.ringbuffer'

local state = GS.new()

local cursor = love.image.newImageData(16,16)
cursor:mapPixel(function(x,y)
	local map = {
	--   0              .  1
	--   1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6
		{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}, -- 00
		{0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,}, --  1
		{0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,}, --  2
		{0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,}, --  3
		{0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,}, --  4
		{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}, --  5
		{0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,}, --  6
		{0,1,1,1,1,0,0,0,0,0,0,1,1,1,1,0,}, --  7
		{0,1,1,1,1,0,0,0,0,0,0,1,1,1,1,0,}, --_ 8
		{0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,}, --  9
		{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}, -- 10
		{0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,}, --  1
		{0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,}, --  2
		{0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,}, --  3
		{0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,}, --  4
		{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,}, --  5
	}
	if map[y+1][x+1] == 1 then
		return 255,255,255,255
	end
	return 0,0,0,0
end)
cursor = love.graphics.newImage(cursor)

local snap = true
local snap_width = 8
local snap_width = 8
local cam, mode
local modes = {}
modes.camera = { image = cursor, pivot = vector() }
function modes.camera:update()
	if love.mouse.isDown('l') then
		local d = vector(love.mouse.getPosition()) - self.pivot
		cam:move(d/10 / cam.zoom)
	else
		self.pivot = vector(love.mouse.getPosition())
	end
end
function modes.camera:draw()
	local font = love.graphics.getFont()
	local function align(str, y)
		love.graphics.print(str, love.graphics.getWidth() - font:getWidth(str) - 10, y)
	end
	align('camera', 5)
	align(('z: %.2f'):format(cam.zoom), 19)
end
function modes.camera:keypressed() end
function modes.camera:mousepressed(x,y,btn)
	if btn == 'wu' then
		cam.zoom = cam.zoom * 1.1
	elseif btn == 'wd' then
		cam.zoom = cam.zoom / 1.1
	end
end
function modes.camera:get() return "" end


modes.entity = { image = cursor, entities = RB('none', 'Human', 'Zombie', 'Shotgun', 'Pistol'), level = {} }
function modes.entity:update()
end
function modes.entity:draw()
	local what = self.entities:get()
	love.graphics.setColor(255,255,255,255)
	if what == 'Human' then
		love.graphics.draw(IMG['Guy.png'], love.mouse.getX()-8,love.mouse.getY()-50, 0,2,2)
	elseif what == 'Zombie' then
		love.graphics.draw(IMG['Zombie.png'], love.mouse.getX()-8,love.mouse.getY()-50, 0,2,2)
	elseif what == 'Shotgun' then
		love.graphics.draw(IMG['shotgun.png'], love.mouse.getX()-16,love.mouse.getY()-25)
	elseif what == 'Pistol' then
		love.graphics.draw(IMG['pistol.png'], love.mouse.getX()-8,love.mouse.getY()-25)
	end
	love.graphics.setColor(255,255,255)
	love.graphics.print(what, 10, love.graphics.getHeight()-20)

	if self.selected then
		cam:predraw()
		self.selected.entity.shape:draw('line')
		cam:postdraw()
	end
end
function modes.entity:keypressed(key)
	if not self.selected then
		return
	end

	if key == 'd' then
		for i = #self.level,1,-1 do
			if self.level[i] == self.selected then
				table.remove(self.level, i)
			end
		end
		Entities.remove(self.selected.entity)
		HC.remove(self.selected.entity.shape)
		self.selected = nil
	end

	local speed = love.keyboard.isDown(' ') and 5 or 1
	if snap then speed = speed * snap_width end
	if key == 'left' then
		self.selected.entity.shape:move(-speed,0)
	elseif key == 'right' then
		self.selected.entity.shape:move(speed,0)
	end
	if key == 'up' then
		self.selected.entity.shape:move(0,-speed)
	elseif key == 'down' then
		self.selected.entity.shape:move(0,speed)
	end
end
function modes.entity:mousepressed(x,y,btn)
	if btn == 'wu' then
		self.entities:next()
	elseif btn == 'wd' then
		self.entities:prev()
	elseif btn == 'r' then
		self.selected = nil
	elseif btn == 'l' then
		--
		-- OH GOD WHAT HAVE I DONE?!
		--
		local what = self.entities:get()
		if what == 'none' then
			if self.selected then
				local x,y = (cam:mousepos()):unpack()
				self.selected.entity.shape:moveTo(x,y)
			else
				for _,o in pairs(self.level) do
					if o.entity.shape:contains( (cam:mousepos()):unpack() ) then
						self.selected = o
						return
					end
				end
			end
		elseif what == 'Human' then
			local h = Human((cam:toWorldCoords(vector(x,y))):unpack())
			self.level[#self.level+1] = { entity = h,
				str = function() return ("Human(%d,%d)"):format(h.shape:center()) end
			}
			self.selected = self.level[#self.level]
		elseif what == 'Zombie' then
			local h = Zombie((cam:toWorldCoords(vector(x,y))):unpack())
			self.level[#self.level+1] = { entity = h,
				str = function() return ("Zombie(%d,%d)"):format(h.shape:center()) end
			}
			self.selected = self.level[#self.level]
		elseif what == 'Shotgun' then
			local x,y = (cam:toWorldCoords(vector(x,y))):unpack()
			local h = Item(x,y,'shotgun')
			self.level[#self.level+1] = { entity = h,
				str = function() return ("Item(%d,%d,'shotgun')"):format(h.shape:center()) end
			}
			self.selected = self.level[#self.level]
		elseif what == 'Pistol' then
			local x,y = (cam:toWorldCoords(vector(x,y))):unpack()
			local h = Item(x,y,'pistol')
			self.level[#self.level+1] = { entity = h,
				str = function() return ("Item(%d,%d,'pistol')"):format(h.shape:center()) end
			}
			self.selected = self.level[#self.level]
		end
	end
end
function modes.entity:get()
	local t = {}
	for i,l in ipairs(self.level) do
		t[i] = l.str()
	end
	return table.concat(t, "\n")
end


modes.geometry = {image = cursor, selected = nil, shapes = {}, mode = 'rect', points = {}}
function modes.geometry:update(dt)
	if self.mode == 'manipulate' and self.selected then
		local speed = love.keyboard.isDown('lshift') and 50 or 10
		if love.keyboard.isDown('left') then
			self.selected.shape:move(-dt * speed, 0)
		elseif love.keyboard.isDown('right') then
			self.selected.shape:move( dt * speed, 0)
		end
		if love.keyboard.isDown('up') then
			self.selected.shape:move(0, -dt * speed)
		elseif love.keyboard.isDown('down') then
			self.selected.shape:move(0, dt * speed)
		end
	end
end
function modes.geometry:draw(skip_legend)
	love.graphics.setColor(255,255,255,128)
	cam:predraw()
	for i,k in ipairs(self.shapes) do
		if k == self.selected then
			k.shape:draw('fill')
		else
			k.shape:draw('line')
		end
	end

	love.graphics.setColor(255,200,150,150)
	if #self.points >= 2 then
		love.graphics.setColor(255,200,150,150)
		for i=1,#self.points,2 do
			local x,y = self.points[i], self.points[i+1]
			love.graphics.circle('fill', x,y, 4,2)
		end
	end

	if self.mode == 'rect' then
		if #self.points >= 2 then
			local p,q = self.points[1], self.points[2]
			local s,t = cam:mousepos():unpack()
			if snap then
				s = math.floor(s / snap_width + .5) * snap_width
				t = math.floor(t / snap_width + .5) * snap_width
			else
				s = math.floor(s)
				t = math.floor(t)
			end
			s = s - p
			t = t - q
			love.graphics.rectangle('line', p,q,s,t)
		end
	elseif self.mode == 'polygon' then
		if #self.points >= 6 then
			love.graphics.polygon('line', unpack(self.points))
		elseif #self.points >= 2 then
			love.graphics.line(self.points)
		end
	end
	love.graphics.setColor(255,255,255)
	cam:postdraw()

	if not skip_legend then
		love.graphics.print('[R]ectangle [P]olygon [D]elete [M]anipulate', 10, love.graphics.getHeight()-20)
		love.graphics.print(self.mode, 10, love.graphics.getHeight()-40)
	end
end
function modes.geometry:keypressed(key)
	if key == 'r' then
		self.mode = 'rect'
		self.points = {}
	elseif key == 'p' then
		self.mode = 'polygon'
		self.points = {}
	elseif key == 'd' and #self.shapes > 0 then
		for i = #self.shapes,1,-1 do
			if self.shapes[i] == self.selected then
				table.remove(self.shapes, i)
				Entities.remove(self.selected.obj, 'background')
			end
		end
		self.points = {}
		self.mode = 'manipulate'
	elseif key == 'm' then
		self.mode = 'manipulate'
		self.points = {}
	end
end
function modes.geometry:mousepressed(x,y,btn)
	if self.mode == 'manipulate' then
		if btn == 'l' then
			for _,s in ipairs(self.shapes) do
				if s.shape:contains( cam:mousepos():unpack() ) then
					self.selected = s
					return
				end
			end
			self.selecetd = nil
		elseif btn == 'wu' and self.selected then
			self.selected.shape:rotate(math.pi/45)
		elseif btn == 'wd' and self.selected then
			self.selected.shape:rotate(math.pi/45)
		end
	elseif self.mode == 'rect' then
		if btn == 'l' then
			local x,y = cam:mousepos():unpack()
			local x,y = cam:mousepos():unpack()
			if snap then
				x = math.floor(x / snap_width + .5) * snap_width
				y = math.floor(y / snap_width + .5) * snap_width
			else
				x = math.floor(x)
				y = math.floor(y)
			end
			self.points[#self.points+1] = x
			self.points[#self.points+1] = y
		elseif btn == 'r' then
			self.points = {}
		end
		if #self.points >= 4 then
			local p,q, s,t = unpack(self.points)
			s = s - p
			t = t - q
			local o = Obstacle(HC.addRectangle(p,q, s,t))
			self.shapes[#self.shapes+1] = {shape = o.shape, obj = o,
				str = function()
					local x,y = o.shape:center()
					return ([[do
    local o = Obstacle(HC.addRectangle(%d,%d,%d,%d))
    o.shape:moveTo(%d,%d)
    o.shape:setRotation(%d)
end]]):format(p,q, s,t, x,y, o.shape:rotation())
			end}
			self.points = {}
		end
	elseif self.mode == 'polygon' then
		if btn == 'l' then
			if love.keyboard.isDown(' ') then
				table.remove(self.points)
				table.remove(self.points)
			else
				local x,y = cam:mousepos():unpack()
				if snap then
					x = math.floor(x / snap_width + .5) * snap_width
					y = math.floor(y / snap_width + .5) * snap_width
				else
					x = math.floor(x)
					y = math.floor(y)
				end
				self.points[#self.points+1] = x
				self.points[#self.points+1] = y
			end
		elseif btn == 'r' and #self.points >= 6 then
			local o = Obstacle(HC.addPolygon(unpack(self.points)))
			self.shapes[#self.shapes+1] = {shape = o.shape,
				str = function()
					local x,y = o.shape:center()
					return ([[do
    local o = Obstacle(HC.addPolygon(%s))
    o.shape:moveTo(%d,%d)
    o.shape:setRotation(%d)
end]]):format(table.concat({o.shape._polygon:unpack()}, ","), x,y, o.shape:rotation())
			end}
			self.points = {}
		end
	end
end
function modes.geometry:get()
	local t = {}
	for i,l in ipairs(self.shapes) do
		t[i] = l.str()
	end
	return table.concat(t, "\n")
end


modes.asset = {image = cursor, selected = nil, assets = {}, where = 'foreground'}
modes.asset.things = RB('ground16','ground32', 'ground64', 'ground128', 'ground256', 'wall32x8', 'wall32x16', 'block32', 'block64', 'block128', 'bed','chair','table','wallpaper','wallpaper_window')
function modes.asset:update(dt)
end
function modes.asset:draw()
	local x,y = love.mouse.getPosition()
	if not self.selected then
		local img = IMG[self.things:get()..'.png']
		local w,h = img:getWidth(), img:getHeight()
		love.graphics.setColor(255,255,255,120)
		if snap then
			x = math.floor(x / snap_width + .5) * snap_width
			y = math.floor(y / snap_width + .5) * snap_width
		else
			x = math.floor(x)
			y = math.floor(y)
		end
		love.graphics.draw(img, x,y, 0,cam.zoom,cam.zoom, w/2,h/2)
	else
		cam:predraw()
		love.graphics.setColor(255,255,255,129)
		love.graphics.circle('fill', self.selected.obj.pos.x, self.selected.obj.pos.y, 6,4)
		cam:postdraw()
	end
end
function modes.asset:keypressed(key)
	if not self.selected then return end
	if #self.assets >= 1 and key == 'd' then
		for i = #self.assets,1,-1 do
			if self.assets[i] == self.selected then
				table.remove(self.assets, i)
				Entities.remove(self.selected.obj, self.selected.where)
			end
		end
		self.selected = nil
	end

	local speed = love.keyboard.isDown('lshift') and 5 or 1
	if snap then speed = speed * snap_width end
	if key == 'up' then
		self.selected.obj.pos.y = self.selected.obj.pos.y - speed
	elseif key == 'down' then
		self.selected.obj.pos.y = self.selected.obj.pos.y + speed
	end
	if key == 'left' then
		self.selected.obj.pos.x = self.selected.obj.pos.x - speed
	elseif key == 'right' then
		self.selected.obj.pos.x = self.selected.obj.pos.x + speed
	end
end
function modes.asset:mousepressed(x,y,btn)
	if btn == 'r' then self.selected = nil end
	if btn == 'l' and love.keyboard.isDown('lshift') then
		local x,y = cam:mousepos():unpack()
		if snap then
			x = math.floor(x / snap_width + .5) * snap_width
			y = math.floor(y / snap_width + .5) * snap_width
		else
			x = math.floor(x)
			y = math.floor(y)
		end
		local what = self.things:get()
		local img = IMG[self.things:get()..'.png']
		local w,h = img:getWidth(), img:getHeight()
		local where = love.keyboard.isDown('lctrl') and 'foreground' or 'background'
		local o = Asset(x,y, img, where)
		self.assets[#self.assets+1] = {obj = o, where = where, str = function()
			return ("Asset(%d,%d, %q, %q, %f)"):format(o.pos.x,o.pos.y, what, where, o.rotation)
		end}
		self.selected = self.assets[#self.assets]
	elseif btn == 'l' and not self.selected then
		for _,a in ipairs(self.assets) do
			local mx,my = cam:mousepos():unpack()
			if a.obj.pos.x-a.obj.img:getWidth()/2 < mx and a.obj.pos.x + a.obj.img:getWidth()/2 > mx and a.obj.pos.y-a.obj.img:getHeight()/2 < my and a.obj.pos.y + a.obj.img:getHeight()/2 > my then
				self.selected = a
				return
			end
		end
		self.selected = nil
		return
	elseif btn == 'wu' then
		if self.selected and love.keyboard.isDown('lshift') then
			self.selected.obj.rotation = self.selected.obj.rotation + math.pi/45
		else
			self.things:next()
		end
	elseif btn == 'wd' then
		if self.selected and love.keyboard.isDown('lshift') then
			self.selected.obj.rotation = self.selected.obj.rotation - math.pi/45
		else
			self.things:prev()
		end
	end
end
function modes.asset:get()
	local t = {}
	for i,l in ipairs(self.assets) do
		t[i] = l.str()
	end
	return table.concat(t, "\n")
end


do
	local _item = Item
	local _zombie = Zombie
	local _human = Human
	local _asset = Asset
	local _obstacle = Obstacle

	function Item(x,y,what)
		local i = _item(x,y,what)
		modes.entity.level[#modes.entity.level+1] = { entity = i,
			str = function() local x,y = h.shape:center(); return ("Item(%d,%d,%q)"):format(x,y,what) end
		}
		return i
	end

	function Zombie(x,y)
		local i = _zombie(x,y)
		modes.entity.level[#modes.entity.level+1] = { entity = i,
			str = function() return ("Zombie(%d,%d)"):format(i.shape:center()) end
		}
		return i
	end

	function Human(x,y)
		local i = _human(x,y)
		modes.entity.level[#modes.entity.level+1] = { entity = i,
			str = function() return ("Human(%d,%d)"):format(i.shape:center()) end
		}
		return i
	end

	function Obstacle(shape)
		local i = _obstacle(shape)
		modes.geometry.shapes[#modes.geometry.shapes+1] = {shape = shape, str = function()
			local x,y = shape:center()
			return ([[do
    local o = Obstacle(HC.addPolygon(%s))
    o.shape:moveTo(%d,%d)
    o.shape:setRotation(%d)
end]]):format(table.concat({shape._polygon:unpack()}, ","), x,y, shape:rotation())
		end}
		return i
	end

	function Asset(x,y, what,where, rot)
		local i = _asset(x,y,what,where,rot)
		modes.asset.assets[#modes.asset.assets+1] = {obj = i, where = where, str = function()
			return ("Asset(%d,%d, %q, %q, %f)"):format(i.pos.x, i.pos.y, what, where, i.rotation)
		end}
		return i
	end
end

function state:enter()
	Entities.clear()
	HC.init(100)
	cam = Camera(vector(0,0), 2)

	mode = modes.camera
	Horde = {}
	Party = {}

	assert(love.filesystem.load('level/level01.lua'))()
end

function state:update(dt)
	mode:update(dt)
end

function state:draw()
	love.graphics.setColor(255,255,255)
	love.graphics.draw(IMG['background.png'], 0,0)
	cam:predraw()
	Entities.draw()
	love.graphics.setColor(255,255,255,100)
	love.graphics.draw(cursor, 0,0, 0,.5,.5, 8,8)
	love.graphics.draw(cursor, 0,0, math.pi/4,2,2, 8,8)
	cam:postdraw()
	love.graphics.setColor(255,255,255)
	love.graphics.draw(mode.image, love.mouse.getX(), love.mouse.getY(), 0, 1,1, mode.image:getWidth()/2, mode.image:getHeight()/2)
	love.graphics.print('[C]amera  [E]entity  [G]eometry  [A]sset  [S]ave  ['..(snap and 'x' or ' ')..']snap', 10,10)

	mode:draw()
	modes.geometry:draw(true)
end

function state:keypressed(key)
	if key == 'c' then
		mode = modes.camera
	elseif key == 'e' then
		mode = modes.entity
	elseif key == 'g' then
		mode = modes.geometry
	elseif key == 'a' then
		mode = modes.asset
	elseif key == ' ' then
		snap = not snap
	elseif key == 's' then
		print(modes.entity:get())
		print()
		print(modes.geometry:get())
		print()
		print(modes.asset:get())
	else
		mode:keypressed(key)
	end
end

function state:mousepressed(...)
	mode:mousepressed(...)
end

return state
