Lights = require 'lights'
require 'collidable'
require 'actors.obstacle'
require 'actors.human'
require 'actors.zombie'
require 'actors.bullet'
require 'actors.weapon'
require 'actors.item'
require 'actors.asset'

Party = { weapons = {} }
Horde = {}

local state = GS.new()

local player, cam

function state:init()
	HC.init(100, function(_,a,b,dx,dy)
		a.object:collideWith(b.object, dx,dy)
		b.object:collideWith(a.object, -dx,-dy)
	end, function(_,a,b,dx,dy)
		a.object:stopCollideWith(b.object, dx,dy)
		b.object:stopCollideWith(a.object, -dx,-dy)
	end)

	function Human.on_die(which)
		table.remove(Party, 1)
		if #Party == 0 then
			player = nil
			GS.switch(State['end'], false)
			return
		end
		if not which.weapon.null_weapon then
			local x,y = which.shape:center()
			Item(x, y+5, which.weapon.img)
		end
		player = Party[1]
		Party[1].player_has_control = true
	end
end

function state:enter(pre, level)
	HC.clear()
	Entities.clear()
	cam = Camera(vector(0,0), 2)
	cam.pan = {x = false, y = false}
	Horde = {}
	Party = { weapons = {} }

	local level = level or 'level01'

	Lights.clear()
	for _,h in ipairs(Party) do
		Lights.lights[h.light] = h.light
	end

	assert(love.filesystem.load('level/'..level..'.lua'))()

	if not player then
		player = Human(0,0,50)
		player.player_has_control = true
		Party[1] = player
	end
	player.shape:moveTo(0,0)

	if not self.src then
		self.src = love.audio.play('snd/drums.ogg', stream, true)
		Timer.add(0, function(f) love.audio.play('snd/pling.ogg') Timer.add(math.random(10,30), f) end)
	end

	function Human:on_goal(goal)
		GS.switch(State['end'], true)
	end
end

function state:update(dt)
	dt = math.min(dt, 1/50)

	Entities.update(dt)

	-- camera panning to the player
	if player then
		local dx,dy = (vector(player.shape:center()) - cam.pos):unpack()
		cam.pan.x = cam.pan.x and math.abs(dx) > 5
		cam.pan.y = cam.pan.y and math.abs(dy) > 5
		if cam.pan.x or math.abs(dx) > 60 then
			cam.pos.x = cam.pos.x + dx * 2 * dt
			cam.pan.x = true
		end
		if cam.pan.y or math.abs(dy) > 60 then
			cam.pos.y = cam.pos.y + dy * 2 * dt
			cam.pan.y = true
		end
	end

	HC.update(dt)
end

function state:draw()
	love.graphics.setColor(255,255,255)
	love.graphics.draw(IMG['background.png'], 0,0)
	cam:predraw()
	Entities.draw()
	Lights.draw(cam)
	cam:postdraw()

	for i,human in ipairs(Party) do
		human:draw_status(10 + (i-1)*35,10)
		Glyphs.print(tostring(i), 12 + (i-1)*35, 18, 8, .4)
	end
	Glyphs.print("STACK:  ", 'right', 10, 10, .4)
	for i,weapon in ipairs(Party.weapons) do
		local scale = case(weapon.img) {
			[IMG['shotgun.png']] = 1,
			default = 1.5
		}
		love.graphics.draw(weapon.img, love.graphics.getWidth()-50, 25 + (i - 1) * 23, 0, scale)
	end
end

function state:keypressed(key)
	if key == 'tab' and #Party.weapons >= 1 then
		if not player.weapon.null_weapon then
			Party.weapons[#Party.weapons+1] = player.weapon
		end
		player.weapon = table.remove(Party.weapons,1)
	else
		local n = tonumber(key)
		if n and n > 1 and n <= #Party then
			Party[1].player_has_control = false
			Party[#Party+1] = Party[1]
			local last = table.remove(Party, 1)
			player = Party[1]
			Party[1].player_has_control = true
			last.following = player
			player.following = nil
		end
	end
end

return state
